@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 # Run this to generate all the initial makefiles, etc.
 
 DIE=0
@@ -9,9 +9,6 @@ srcfile=gst/gst.c
 if test ! -d common; then
   echo "+ getting common from cvs"; cvs co common
 fi
-if test ! -d libs/ext/cothreads; then
-  echo "+ getting cothreads from cvs"; cvs co gst-cothreads
-fi
 
 # source helper functions
 if test ! -f common/gst-autogen.sh;
@@ -22,24 +19,26 @@ then
 fi
 . common/gst-autogen.sh
 
-CONFIGURE_DEF_OPT='--enable-maintainer-mode --enable-plugin-builddir --enable-failing-tests'
+CONFIGURE_DEF_OPT='--enable-maintainer-mode --enable-plugin-builddir --enable-failing-tests --enable-poisoning'
 
 autogen_options $@
 
 echo -n "+ check for build tools"
-if test ! -z $NOCHECK; then echo ": skipped version checks"; else  echo; fi
+if test ! -z "$NOCHECK"; then echo ": skipped version checks"; else  echo; fi
 version_check "autoconf" "$AUTOCONF autoconf autoconf-2.54 autoconf-2.53 autoconf-2.52" \
               "ftp://ftp.gnu.org/pub/gnu/autoconf/" 2 52 || DIE=1
-version_check "automake" "$AUTOMAKE automake automake-1.7 automake-1.6 automake-1.5" \
-              "ftp://ftp.gnu.org/pub/gnu/automake/" 1 5 || DIE=1
-version_check "libtool" "" \
+version_check "automake" "$AUTOMAKE automake automake-1.7 automake17 automake-1.6" \
+              "ftp://ftp.gnu.org/pub/gnu/automake/" 1 6 || DIE=1
+version_check "autopoint" "autopoint" \
+              "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 11 4 || DIE=1
+version_check "libtoolize" "libtoolize libtoolize14" \
               "ftp://ftp.gnu.org/pub/gnu/libtool/" 1 4 0 || DIE=1
 version_check "pkg-config" "" \
               "http://www.freedesktop.org/software/pkgconfig" 0 8 0 || DIE=1
 
 die_check $DIE
 
-autoconf_2.52d_check || DIE=1
+autoconf_2_52d_check || DIE=1
 aclocal_check || DIE=1
 autoheader_check || DIE=1
 
@@ -56,10 +55,22 @@ fi
 
 toplevel_check $srcfile
 
-if test -e acinclude.m4; then rm acinclude.m4; fi
+# autopoint
+#    older autopoint (< 0.12) has a tendency to complain about mkinstalldirs
+if test -e mkinstalldirs; then rm mkinstalldirs; fi
+#    first remove patch if necessary, then run autopoint, then reapply
+if test -f po/Makefile.in.in;
+then
+  patch -p0 -R < common/gettext.patch
+fi
+tool_run "$autopoint --force"
+patch -p0 < common/gettext.patch
+
+# aclocal
+if test -f acinclude.m4; then rm acinclude.m4; fi
 tool_run "$aclocal" "-I common/m4 $ACLOCAL_FLAGS"
 
-tool_run "libtoolize" "--copy --force"
+tool_run "$libtoolize" "--force"
 tool_run "$autoheader"
 
 # touch the stamp-h.in build stamp so we don't re-run autoheader in maintainer mode -- wingo
@@ -69,17 +80,6 @@ tool_run "$autoconf"
 debug "automake: $automake"
 tool_run "$automake" "-a -c"
 
-echo
-echo "+ running autogen.sh --noconfigure $@ in libs/ext/cothreads..."
-pushd libs/ext/cothreads > /dev/null
-echo
-./autogen.sh --noconfigure $@ || {
-        echo "autogen in cothreads failed."
-        exit 1
-}
-popd > /dev/null
-echo
-
 test -n "$NOCONFIGURE" && {
   echo "skipping configure stage for package $package, as requested."
   echo "autogen.sh done."
