@@ -26,6 +26,7 @@
 #include "gstregistrypool.h"
 #include "gstpad.h"
 #include "gstindex.h"
+#include "gstmarshal.h"
 
 /* Index signals and args */
 enum {
@@ -35,7 +36,7 @@ enum {
 
 enum {
   ARG_0,
-  ARG_RESOLVER,
+  ARG_RESOLVER
   /* FILL ME */
 };
 
@@ -88,6 +89,20 @@ gst_index_resolver_get_type (void)
   return index_resolver_type;
 }
 
+GType
+gst_index_entry_get_type (void)
+{
+  static GType index_entry_type = 0;
+
+  if (!index_entry_type) {
+    index_entry_type = g_boxed_type_register_static ("GstIndexEntry",
+        (GBoxedCopyFunc) gst_index_entry_copy,
+        (GBoxedFreeFunc) gst_index_entry_free);
+  }
+  return index_entry_type;
+}
+
+
 GType
 gst_index_get_type (void) {
   static GType index_type = 0;
@@ -381,6 +396,18 @@ gst_index_set_resolver (GstIndex *index,
   index->method = GST_INDEX_RESOLVER_CUSTOM;
 }
 
+/**
+ * gst_index_entry_copy:
+ * @entry: the entry to copy
+ *
+ * Copies an entry and returns the result.
+ */
+GstIndexEntry *
+gst_index_entry_copy (GstIndexEntry *entry)
+{
+  return g_memdup(entry, sizeof(*entry));
+}
+
 /**
  * gst_index_entry_free:
  * @entry: the entry to free
@@ -684,7 +711,7 @@ gst_index_compare_func (gconstpointer a,
                         gconstpointer b,
                         gpointer user_data)
 {
-  return a - b;  
+  return (gint)a - (gint)b;  
 }
 
 /**
