@@ -28,10 +28,24 @@
 #include <gst/gst.h>
 #include <gobject/gvaluecollector.h>
 
-//#define G_TYPE_FOURCC G_TYPE_FLOAT
-
 static GType _gst_structure_type;
 
+typedef struct _GstStructureField GstStructureField;
+struct _GstStructureField {
+  GQuark name;
+  GValue value;
+};
+
+#define GST_STRUCTURE_FIELD(structure, index) \
+    &g_array_index((structure)->fields, GstStructureField, (index))
+
+static void gst_structure_set_field (GstStructure *structure,
+    GstStructureField *field);
+static GstStructureField *gst_structure_get_field(const GstStructure *structure,
+        const gchar *fieldname);
+static GstStructureField *gst_structure_id_get_field(const GstStructure *structure,
+        GQuark field);
+
 static void _gst_structure_transform_to_string(const GValue *src_value,
     GValue *dest_value);
 
@@ -85,7 +99,7 @@ void _gst_structure_initialize(void)
 
 /**
  * gst_structure_id_empty_new:
- * @name: name of new structure
+ * @quark: name of new structure
  *
  * Creates a new, empty #GstStructure with the given name.
  *
@@ -159,7 +173,7 @@ GstStructure *gst_structure_new(const gchar *name,
  * gst_structure_new_valist:
  * @name: name of new structure
  * @firstfield: name of first field to set
- * @varags: variable argument list
+ * @varargs: variable argument list
  *
  * Creates a new #GstStructure with the given name.  Structure fields
  * are set according to the varargs in a manner similar to
@@ -197,8 +211,6 @@ GstStructure *gst_structure_copy(GstStructure *structure)
   g_return_val_if_fail(structure != NULL, NULL);
 
   new_structure = gst_structure_empty_new(g_quark_to_string(structure->name));
-  new_structure->fields = g_array_set_size(new_structure->fields,
-      structure->fields->len);
   new_structure->name = structure->name;
 
   for(i=0;i<structure->fields->len;i++){
@@ -207,13 +219,11 @@ GstStructure *gst_structure_copy(GstStructure *structure)
     field = GST_STRUCTURE_FIELD(structure, i);
 
     new_field.name = field->name;
-    g_value_init(&new_field.value, G_VALUE_TYPE(&field->value));
-    g_value_copy(&field->value, &new_field.value);
-
+    gst_value_init_and_copy (&new_field.value, &field->value);
     g_array_append_val(new_structure->fields, new_field);
   }
 
-  return structure;
+  return new_structure;
 }
 
 /**
@@ -238,6 +248,9 @@ void gst_structure_free(GstStructure *structure)
       g_value_unset(&field->value);
     }
   }
+#ifdef USE_POISONING
+  memset (structure, 0xff, sizeof(GstStructure));
+#endif
   g_free(structure);
 }
 
@@ -249,7 +262,7 @@ void gst_structure_free(GstStructure *structure)
  *
  * Returns: the name of the structure.
  */
-const gchar *gst_structure_get_name(GstStructure *structure)
+const gchar *gst_structure_get_name(const GstStructure *structure)
 {
   g_return_val_if_fail(structure != NULL, NULL);
 
@@ -275,52 +288,51 @@ void gst_structure_set_name(GstStructure *structure, const gchar *name)
 /**
  * gst_structure_id_set_value:
  * @structure: a #GstStructure
- * @field_id: a #GQuark representing a field
+ * @field: a #GQuark representing a field
  * @value: the new value of the field
  *
  * Sets the field with the given ID to the provided value.  If the field
  * does not exist, it is created.  If the field exists, the previous
  * value is freed.
  */
-void gst_structure_id_set_value(GstStructure *structure, GQuark fieldname,
+void gst_structure_id_set_value(GstStructure *structure, GQuark field,
     const GValue *value)
 {
-  GstStructureField field = { 0, { 0, } };
+  GstStructureField gsfield = { 0, { 0, } };
 
   g_return_if_fail(structure != NULL);
   g_return_if_fail(G_IS_VALUE(value));
 
-  field.name = fieldname;
-  g_value_init(&field.value, G_VALUE_TYPE (value));
-  g_value_copy(value, &field.value);
+  gsfield.name = field;
+  gst_value_init_and_copy (&gsfield.value, value);
 
-  gst_structure_set_field(structure, &field);
+  gst_structure_set_field(structure, &gsfield);
 }
 
 /**
  * gst_structure_set_value:
  * @structure: a #GstStructure
- * @field: the name of the field to set
+ * @fieldname: the name of the field to set
  * @value: the new value of the field
  *
  * Sets the field with the given name to the provided value.  If the field
  * does not exist, it is created.  If the field exists, the previous
  * value is freed.
  */
-void gst_structure_set_value(GstStructure *structure, const gchar *field,
+void gst_structure_set_value(GstStructure *structure, const gchar *fieldname,
     const GValue *value)
 {
   g_return_if_fail(structure != NULL);
-  g_return_if_fail(field != NULL);
+  g_return_if_fail(fieldname != NULL);
   g_return_if_fail(G_IS_VALUE(value));
 
-  gst_structure_id_set_value(structure, g_quark_from_string(field), value);
+  gst_structure_id_set_value(structure, g_quark_from_string(fieldname), value);
 }
 
 /**
  * gst_structure_set:
  * @structure: a #GstStructure
- * @field: the name of the field to set
+ * @fieldname: the name of the field to set
  * @...: variable arguments
  *
  * Parses the variable arguments and sets fields accordingly.
@@ -341,9 +353,9 @@ void gst_structure_set(GstStructure *structure, const gchar *field, ...)
 }
 
 /**
- * gst_structure_set:
+ * gst_structure_set_valist:
  * @structure: a #GstStructure
- * @field: the name of the field to set
+ * @fieldname: the name of the field to set
  * @varargs: variable arguments
  *
  * va_list form of #gst_structure_set.
@@ -391,9 +403,21 @@ void gst_structure_set_valist(GstStructure *structure, const gchar *fieldname,
 	  i = va_arg(varargs, int);
 	  g_value_init(&field.value, GST_TYPE_FOURCC);
 	  gst_value_set_fourcc(&field.value, i);
-	  break;
-	}else{
+        } else if (type == GST_TYPE_INT_RANGE){
+          int min, max;
+	  min = va_arg(varargs, int);
+	  max = va_arg(varargs, int);
+	  g_value_init(&field.value, GST_TYPE_INT_RANGE);
+	  gst_value_set_int_range(&field.value, min, max);
+        } else if (type == GST_TYPE_DOUBLE_RANGE){
+          double min, max;
+	  min = va_arg(varargs, double);
+	  max = va_arg(varargs, double);
+	  g_value_init(&field.value, GST_TYPE_DOUBLE_RANGE);
+	  gst_value_set_double_range(&field.value, min, max);
+	} else {
 	  g_critical("unimplemented vararg field type %d\n", (int)type);
+          return;
 	}
 	break;
     }
@@ -404,32 +428,6 @@ void gst_structure_set_valist(GstStructure *structure, const gchar *fieldname,
   }
 }
 
-/**
- * gst_structure_set_field_copy:
- * @structure: a #GstStructure
- * @field: the #GstStructureField to set
- *
- * Sets a field in the structure.  If the structure currently contains
- * a field with the same name, it is replaced with the provided field.
- * Otherwise, the field is added to the structure.  The field's value
- * is deeply copied.
- *
- * This function is intended mainly for internal use.  The function
- * #gst_structure_set() is recommended instead of this one.
- */
-void gst_structure_set_field_copy (GstStructure *structure,
-    const GstStructureField *field)
-{
-  GstStructureField f = { 0 };
-  GType type = G_VALUE_TYPE (&field->value);
-
-  f.name = field->name;
-  g_value_init (&f.value, type);
-  g_value_copy (&field->value, &f.value);
-
-  gst_structure_set_field (structure, &f);
-}
-
 /**
  * gst_structure_set_field:
  * @structure: a #GstStructure
@@ -443,7 +441,7 @@ void gst_structure_set_field_copy (GstStructure *structure,
  * This function is intended mainly for internal use.  The function
  * #gst_structure_set() is recommended instead of this one.
  */
-void gst_structure_set_field(GstStructure *structure, GstStructureField *field)
+static void gst_structure_set_field(GstStructure *structure, GstStructureField *field)
 {
   GstStructureField *f;
   int i;
@@ -461,7 +459,7 @@ void gst_structure_set_field(GstStructure *structure, GstStructureField *field)
   g_array_append_val(structure->fields, *field);
 }
 
-/**
+/* FIXME: is this private ? if so remove gtk-doc
  * gst_structure_id_get_field:
  * @structure: a #GstStructure
  * @field_id: the GQuark of the field to get
@@ -471,7 +469,7 @@ void gst_structure_set_field(GstStructure *structure, GstStructureField *field)
  *
  * Returns: the #GstStructureField with the given ID
  */
-GstStructureField *gst_structure_id_get_field(const GstStructure *structure,
+static GstStructureField *gst_structure_id_get_field(const GstStructure *structure,
     GQuark field_id)
 {
   GstStructureField *field;
@@ -498,7 +496,7 @@ GstStructureField *gst_structure_id_get_field(const GstStructure *structure,
  *
  * Returns: the #GstStructureField with the given name
  */
-GstStructureField *
+static GstStructureField *
 gst_structure_get_field(const GstStructure *structure, const gchar *fieldname)
 {
   g_return_val_if_fail(structure != NULL, NULL);
@@ -531,12 +529,28 @@ gst_structure_get_value(const GstStructure *structure, const gchar *fieldname)
   return &field->value;
 }
 
-#if 0
-void gst_structure_get(GstStructure *structure, const gchar *fieldname, ...)
+/**
+ * gst_structure_id_get_value:
+ * @structure: a #GstStructure
+ * @field: the #GQuark of the field to get
+ *
+ * Accessor function.
+ *
+ * Returns: the #GValue corresponding to the field with the given name 
+ *          identifier.
+ */
+const GValue *
+gst_structure_id_get_value(const GstStructure *structure, GQuark field)
 {
+  GstStructureField *gsfield;
 
+  g_return_val_if_fail(structure != NULL, NULL);
+
+  gsfield = gst_structure_id_get_field(structure, field);
+  if(gsfield == NULL) return NULL;
+
+  return &gsfield->value;
 }
-#endif
 
 /**
  * gst_structure_remove_field:
@@ -571,6 +585,53 @@ gst_structure_remove_field(GstStructure *structure, const gchar *fieldname)
   }
 }
 
+/**
+ * gst_structure_remove_fields:
+ * @structure: a #GstStructure
+ * @fieldname: the name of the field to remove
+ * @...: NULL-terminated list of more fieldnames to remove
+ *
+ * Removes the field with the given names. If a field does not exist, the
+ * argument is ignored.
+ */
+void 
+gst_structure_remove_fields(GstStructure *structure, const gchar *fieldname, ...)
+{
+  va_list varargs;
+
+  g_return_if_fail (structure != NULL);
+  g_return_if_fail (fieldname != NULL);
+
+  va_start(varargs, fieldname);
+
+  gst_structure_remove_fields_valist (structure, fieldname, varargs);
+
+  va_end(varargs);
+}
+
+/**
+ * gst_structure_remove_fields_valist:
+ * @structure: a #GstStructure
+ * @fieldname: the name of the field to remove
+ * @varargs: NULL-terminated list of more fieldnames to remove
+ *
+ * Removes the field with the given names. If a field does not exist, the
+ * argument is ignored.
+ */
+void 
+gst_structure_remove_fields_valist(GstStructure *structure, 
+    const gchar *fieldname, va_list varargs)
+{
+  gchar *field = (gchar *) fieldname;
+  g_return_if_fail (structure != NULL);
+  g_return_if_fail (fieldname != NULL);
+
+  while (field) {
+    gst_structure_remove_field (structure, field);
+    field = va_arg (varargs, char *); 
+  }
+}
+
 /**
  * gst_structure_remove_all_fields:
  * @structure: a #GstStructure
@@ -637,25 +698,32 @@ gst_structure_n_fields(const GstStructure *structure)
 }
 
 /**
- * gst_structure_field_foreach:
+ * gst_structure_foreach:
  * @structure: a #GstStructure
  * @func: a function to call for each field
  * @user_data: private data
  *
  * Calls the provided function once for each field in the #GstStructure.
+ *
+ * Returns: TRUE if the supplied function returns TRUE For each of the fields,
+ * FALSE otherwise.
  */
-void
-gst_structure_field_foreach (GstStructure *structure,
+gboolean
+gst_structure_foreach (GstStructure *structure,
     GstStructureForeachFunc func, gpointer user_data)
 {
   int i;
   GstStructureField *field;
+  gboolean ret;
 
   for(i=0;i<structure->fields->len;i++){
     field = GST_STRUCTURE_FIELD(structure, i);
 
-    func (structure, field->name, &field->value, user_data);
+    ret = func (field->name, &field->value, user_data);
+    if (!ret) return FALSE;
   }
+
+  return TRUE;
 }
 
 /**
@@ -681,7 +749,7 @@ gst_structure_has_field(const GstStructure *structure, const gchar *fieldname)
 }
 
 /**
- * gst_structure_has_field:
+ * gst_structure_has_field_typed:
  * @structure: a #GstStructure
  * @fieldname: the name of a field
  * @type: the type of a value
@@ -712,9 +780,9 @@ gst_structure_has_field_typed(const GstStructure *structure, const gchar *fieldn
  * gst_structure_get_boolean:
  * @structure: a #GstStructure
  * @fieldname: the name of a field
- * @ptr: a pointer to a #gboolean to set
+ * @value: a pointer to a #gboolean to set
  *
- * Sets the boolean pointed to by @ptr corresponding to the value of the
+ * Sets the boolean pointed to by @value corresponding to the value of the
  * given field.  Caller is responsible for making sure the field exists
  * and has the correct type.
  *
@@ -743,9 +811,9 @@ gst_structure_get_boolean(const GstStructure *structure, const gchar *fieldname,
  * gst_structure_get_int:
  * @structure: a #GstStructure
  * @fieldname: the name of a field
- * @ptr: a pointer to an int to set
+ * @value: a pointer to an int to set
  *
- * Sets the int pointed to by @ptr corresponding to the value of the
+ * Sets the int pointed to by @value corresponding to the value of the
  * given field.  Caller is responsible for making sure the field exists
  * and has the correct type.
  *
@@ -775,9 +843,9 @@ gst_structure_get_int(const GstStructure *structure, const gchar *fieldname,
  * gst_structure_get_fourcc:
  * @structure: a #GstStructure
  * @fieldname: the name of a field
- * @ptr: a pointer to a #GstFourcc to set
+ * @value: a pointer to a #GstFourcc to set
  *
- * Sets the #GstFourcc pointed to by @ptr corresponding to the value of the
+ * Sets the #GstFourcc pointed to by @value corresponding to the value of the
  * given field.  Caller is responsible for making sure the field exists
  * and has the correct type.
  *
@@ -796,9 +864,9 @@ gst_structure_get_fourcc(const GstStructure *structure, const gchar *fieldname,
   field = gst_structure_get_field(structure, fieldname);
 
   if(field == NULL) return FALSE;
-  if(!G_VALUE_HOLDS_UINT(&field->value))return FALSE;
+  if(!GST_VALUE_HOLDS_FOURCC(&field->value))return FALSE;
 
-  *value = g_value_get_uint(&field->value);
+  *value = gst_value_get_fourcc (&field->value);
 
   return TRUE;
 }
@@ -807,9 +875,9 @@ gst_structure_get_fourcc(const GstStructure *structure, const gchar *fieldname,
  * gst_structure_get_double:
  * @structure: a #GstStructure
  * @fieldname: the name of a field
- * @ptr: a pointer to a #GstFourcc to set
+ * @value: a pointer to a #GstFourcc to set
  *
- * Sets the double pointed to by @ptr corresponding to the value of the
+ * Sets the double pointed to by @value corresponding to the value of the
  * given field.  Caller is responsible for making sure the field exists
  * and has the correct type.
  *
@@ -838,7 +906,6 @@ gboolean gst_structure_get_double(const GstStructure *structure,
  * gst_structure_get_string:
  * @structure: a #GstStructure
  * @fieldname: the name of a field
- * @ptr: a pointer to a #GstFourcc to set
  *
  * Finds the field corresponding to @fieldname, and returns the string
  * contained in the field's value.  Caller is responsible for making
@@ -926,6 +993,10 @@ static const char *_gst_structure_to_abbr(GType type)
   return g_type_name(type);
 }
 
+#define GST_ASCII_IS_STRING(c) (g_ascii_isalnum((c)) || ((c) == '_') || \
+      ((c) == '-') || ((c) == '+') || ((c) == '/') || ((c) == ':') || \
+      ((c) == '.'))
+
 /**
  * gst_structure_to_string:
  * @structure: a #GstStructure
@@ -940,26 +1011,31 @@ gst_structure_to_string(const GstStructure *structure)
   GstStructureField *field;
   GString *s;
   int i;
+  
+  /* NOTE:  This function is potentially called by the debug system,
+   * so any calls to gst_log() (and GST_DEBUG(), GST_LOG(), etc.)
+   * should be careful to avoid recursion.  This includes any functions
+   * called by gst_structure_to_string.  In particular, calls should
+   * not use the GST_PTR_FORMAT extension.  */
 
   g_return_val_if_fail(structure != NULL, NULL);
 
   s = g_string_new("");
-  g_string_append_printf(s, "\"%s\"", g_quark_to_string(structure->name));
-  for(i=0;i<structure->fields->len;i++){
-    GValue s_val = { 0 };
+  /* FIXME this string may need to be escaped */
+  g_string_append_printf(s, "%s", g_quark_to_string(structure->name));
+  for(i=0;i<structure->fields->len;i++) {
+    char *t;
     GType type;
 
     field = GST_STRUCTURE_FIELD(structure, i);
 
-    g_value_init(&s_val, G_TYPE_STRING);
-    g_value_transform (&field->value, &s_val);
-
+    t = gst_value_serialize (&field->value);
     type = G_VALUE_TYPE (&field->value);
 
     if (type == GST_TYPE_LIST) {
-      GPtrArray *ptr_array = g_value_peek_pointer (&field->value);
-      if (ptr_array->len > 0){
-        GValue *value = g_ptr_array_index (ptr_array, 0);
+      GArray *array = g_value_peek_pointer (&field->value);
+      if (array->len > 0){
+        GValue *value = &g_array_index (array, GValue, 0);
 
 	type = G_VALUE_TYPE (value);
       } else {
@@ -970,9 +1046,9 @@ gst_structure_to_string(const GstStructure *structure)
     } else if (G_VALUE_TYPE(&field->value) == GST_TYPE_DOUBLE_RANGE) {
       type = G_TYPE_DOUBLE;
     }
-    g_string_append_printf(s, ", %s:%s=%s", g_quark_to_string(field->name),
-	_gst_structure_to_abbr(type), g_value_get_string(&s_val));
-    g_value_unset (&s_val);
+    g_string_append_printf(s, ", %s=(%s)%s", g_quark_to_string(field->name),
+	_gst_structure_to_abbr(type), t);
+    g_free(t);
   }
   return g_string_free(s, FALSE);
 }
@@ -984,216 +1060,87 @@ gst_structure_to_string(const GstStructure *structure)
  * unread data.
  * THIS FUNCTION MODIFIES THE STRING AND DETECTS INSIDE A NONTERMINATED STRING 
  */
+static gboolean _gst_structure_parse_simple_string (gchar *s, gchar **end);
 static gboolean
-_gst_structure_parse_string (gchar *r, gchar **end, gchar **next)
+_gst_structure_parse_string (gchar *s, gchar **end, gchar **next)
 {
   gchar *w;
-  gchar c = '\0';
 
-  w = r;
-  if (*r == '\'' || *r == '\"') {
-    c = *r;
-    r++;
+  if (*s == 0) return FALSE;
+
+  if (*s != '"') {
+    int ret;
+
+    ret = _gst_structure_parse_simple_string (s, end);
+    *next = *end;
+
+    return ret;
   }
 
-  for (;;r++) {
-    if (*r == '\0') {
-      if (c) {
-        goto error;
-      } else {
-        goto found;
-      }
+  w = s;
+  s++;
+  while (*s != '"') {
+    if (*s == 0) return FALSE;
+
+    if (*s == '\\') {
+      s++;
     }
 
-    if (*r == '\\') {
-      r++;
-      if (*r == '\0')
-        goto error;
-      *w++ = *r;
-      continue;
-    }
-
-    if (*r == c) {
-      r++;
-      if (*r == '\0')
-        goto found;
-      break;
-    }
-
-    if (!c) {
-      if (g_ascii_isspace (*r))
-        break;
-      /* this needs to be escaped */
-      if (*r == ',' || *r == ')' || *r == ']' || *r == ':' ||
-          *r == ';' || *r == '(' || *r == '[')
-        break;
-    }
-    *w++ = *r;
+    *w = *s;
+    w++;
+    s++;
   }
+  s++;
 
-found:
-  while (g_ascii_isspace (*r)) r++;
-  if (w != r)
-    *w++ = '\0';
   *end = w;
-  *next = r;
+  *next = s;
+
   return TRUE;
-
-error:
-  return FALSE;
 }
 
-static gboolean
-_gst_structure_parse_value (gchar *s, gchar **after, GType type,
-    GValue *value)
-{
-  gboolean ret = FALSE;
-  gchar *val;
-  gchar *end;
-
-  if (type == G_TYPE_INVALID) return FALSE;
-
-  while (g_ascii_isspace (*s)) s++;
-
-  g_value_init(value, type);
-
-  val = s;
-  switch (type) {
-    case G_TYPE_INT:
-      {
-	int x;
-	x = strtol (val, &s, 0);
-	if (val != s) {
-	  g_value_set_int (value, x);
-	  ret = TRUE;
-	}
-      }
-      break;
-    case G_TYPE_FLOAT:
-      {
-	double x;
-	x = g_ascii_strtod (val, &s);
-	if (val != s) {
-	  g_value_set_float (value, x);
-	  ret = TRUE;
-	}
-      }
-      break;
-    case G_TYPE_DOUBLE:
-      {
-	double x;
-	x = g_ascii_strtod (val, &s);
-	if (val != s) {
-	  g_value_set_double (value, x);
-	  ret = TRUE;
-	}
-      }
-      break;
-    case G_TYPE_BOOLEAN:
-      {
-	int len;
-
-	ret = _gst_structure_parse_string (val, &end, &s);
-	len = end - val;
-	if (ret && len > 0) {
-	  if (g_ascii_strncasecmp (val, "true", len) == 0 ||
-	      g_ascii_strncasecmp (val, "yes", len) == 0 ||
-	      g_ascii_strncasecmp (val, "t", len) == 0 ||
-	      strncmp (val, "1", len)) {
-	    g_value_set_boolean (value, TRUE);
-	  } else if (g_ascii_strncasecmp (val, "false", len) == 0 ||
-	      g_ascii_strncasecmp (val, "no", len) == 0 ||
-	      g_ascii_strncasecmp (val, "f", len) == 0 ||
-	      strncmp (val, "0", len)) {
-	    g_value_set_boolean (value, FALSE);
-	  } else {
-	    ret = FALSE;
-	  }
-	}
-      }
-      break;
-    case G_TYPE_STRING:
-      {
-	ret = _gst_structure_parse_string (val, &end, &s);
-	if (ret) {
-	  g_value_set_string_take_ownership (value,
-	      g_strndup(val, end - val));
-	  ret = TRUE;
-	}
-      }
-      break;
-    default:
-      /* FIXME: make more general */
-      if (type == GST_TYPE_FOURCC) {
-	guint32 fourcc = 0;
-	if (g_ascii_isdigit (*s)) {
-	  fourcc = strtoul (val, &s, 0);
-	  if (val != s) {
-	    ret = TRUE;
-	  }
-	} else {
-	  ret = _gst_structure_parse_string (val, &end, &s);
-g_print("end - val = %d\n", end - val);
-	  if (end - val >= 4) {
-	    fourcc = GST_MAKE_FOURCC(val[0], val[1], val[2], val[3]);
-	    ret = TRUE;
-	  }
-	}
-	gst_value_set_fourcc (value, fourcc);
-      } else {
-	g_critical("type %s not handled", g_type_name(type));
-      }
-      break;
-  }
-
-  *after = s;
-
-  return ret;
-}
+static gboolean _gst_structure_parse_value (gchar *str, gchar **after,
+    GValue *value, GType default_type);
 
 static gboolean
-_gst_structure_parse_range (gchar *s, gchar **after, GType type,
-    GValue *value)
+_gst_structure_parse_range (gchar *s, gchar **after, GValue *value, GType type)
 {
   GValue value1 = { 0 };
   GValue value2 = { 0 };
   GType range_type;
   gboolean ret;
 
-  if (type == G_TYPE_DOUBLE) {
-    range_type = GST_TYPE_DOUBLE_RANGE;
-  } else if (type == G_TYPE_INT) {
-    range_type = GST_TYPE_INT_RANGE;
-  } else {
-    return FALSE;
-  }
 
-g_print("%d \"%s\"\n", __LINE__, s);
   if (*s != '[') return FALSE;
   s++;
 
-g_print("%d \"%s\"\n", __LINE__, s);
-  ret = _gst_structure_parse_value(s, &s, type, &value1);
+  ret = _gst_structure_parse_value(s, &s, &value1, type);
   if (ret == FALSE) return FALSE;
 
   while (g_ascii_isspace (*s)) s++;
     
-g_print("%d \"%s\"\n", __LINE__, s);
   if (*s != ',') return FALSE;
   s++;
 
   while (g_ascii_isspace (*s)) s++;
 
-g_print("%d \"%s\"\n", __LINE__, s);
-  ret = _gst_structure_parse_value(s, &s, type, &value2);
+  ret = _gst_structure_parse_value(s, &s, &value2, type);
   if (ret == FALSE) return FALSE;
 
   while (g_ascii_isspace (*s)) s++;
 
-g_print("%d \"%s\"\n", __LINE__, s);
   if (*s != ']') return FALSE;
   s++;
 
+  if (G_VALUE_TYPE (&value1) != G_VALUE_TYPE (&value2)) return FALSE;
+  
+  if (G_VALUE_TYPE (&value1) == G_TYPE_DOUBLE) {
+    range_type = GST_TYPE_DOUBLE_RANGE;
+  } else if (G_VALUE_TYPE (&value1) == G_TYPE_INT) {
+    range_type = GST_TYPE_INT_RANGE;
+  } else {
+    return FALSE;
+  }
+
   g_value_init(value, range_type);
   if (range_type == GST_TYPE_DOUBLE_RANGE) {
     gst_value_set_double_range(value, g_value_get_double(&value1),
@@ -1208,38 +1155,43 @@ g_print("%d \"%s\"\n", __LINE__, s);
 }
 
 static gboolean
-_gst_structure_parse_list (gchar *s, gchar **after, GType type, GValue *value)
+_gst_structure_parse_list (gchar *s, gchar **after, GValue *value, GType type)
 {
   GValue list_value = { 0 };
   gboolean ret;
+  GArray *array;
 
   g_value_init(value, GST_TYPE_LIST);
+  array = g_value_peek_pointer (value);
 
-  if (*s != '(') return FALSE;
+  if (*s != '{') return FALSE;
   s++;
 
   while (g_ascii_isspace (*s)) s++;
-  if (*s == ')') {
+  if (*s == '}') {
     s++;
     *after = s;
     return TRUE;
   }
 
-  ret = _gst_structure_parse_value(s, &s, type, &list_value);
+  ret = _gst_structure_parse_value(s, &s, &list_value, type);
   if (ret == FALSE) return FALSE;
+  
+  g_array_append_val (array, list_value);
 
   while (g_ascii_isspace (*s)) s++;
     
-  while (*s != ')') {
+  while (*s != '}') {
     if (*s != ',') return FALSE;
     s++;
 
     while (g_ascii_isspace (*s)) s++;
 
     memset (&list_value, 0, sizeof (list_value));
-    ret = _gst_structure_parse_value(s, &s, type, &list_value);
+    ret = _gst_structure_parse_value(s, &s, &list_value, type);
     if (ret == FALSE) return FALSE;
 
+    g_array_append_val (array, list_value);
     while (g_ascii_isspace (*s)) s++;
   }
 
@@ -1250,67 +1202,117 @@ _gst_structure_parse_list (gchar *s, gchar **after, GType type, GValue *value)
 }
 
 static gboolean
-_gst_structure_parse_field (gchar *str, gchar **after, GstStructureField *field)
+_gst_structure_parse_simple_string (gchar *str, gchar **end)
 {
-  /* NAME[:TYPE]=VALUE */
-  gchar *name;
-  gchar *type_name;
-  gchar *s, *del;
-  gboolean have_type = FALSE;
-  GType type = G_TYPE_INVALID;
-  int ret;
+  char *s = str;
 
-g_print("parsing: \"%s\"\n", str);
-  name = s = str;
-  while (g_ascii_isalnum (*s) || *s == '_' || *s == '-') s++;
-  del = s;
-  while (g_ascii_isspace (*s)) s++;
-  if (!(*s == '=' || *s == ':')) return FALSE;
-  if (*s == ':') have_type = TRUE;
-  s++;
-  while (g_ascii_isspace (*s)) s++;
-  *del = '\0';
-
-  field->name = g_quark_from_string (name);
-
-  if (have_type) {
-    while (g_ascii_isspace (*s)) s++;
-    type_name = s;
-    while (g_ascii_isalnum (*s) || *s == '_' || *s == '-') s++;
-    del = s;
-    while (g_ascii_isspace (*s)) s++;
-    if (*s != '=') return FALSE;
+  while(GST_ASCII_IS_STRING(*s)){
     s++;
-    while (g_ascii_isspace (*s)) s++;
-    *del = '\0';
-  
-g_print("type name is \"%s\"\n",type_name);
-    type = _gst_structure_from_abbr(type_name);
-g_print("type n is \"%s\"\n",g_type_name(type));
-
-    if (type == G_TYPE_INVALID) return FALSE;
-
-  } else {
-    if (g_ascii_isdigit (*s) ||
-	((*s == '-' || *s == '+') && g_ascii_isdigit (s[1]))) {
-      char *t = s;
-      while (g_ascii_isdigit (*t)) t++;
-      if (*t == '.'){
-        type = G_TYPE_DOUBLE;
-      } else {
-        type = G_TYPE_INT;
-      }
-    } else if (g_ascii_isalpha (*s) || *s == '"' || *s == '\'') {
-      type = G_TYPE_STRING;
-    }
   }
 
+  *end = s;
+
+  return (s != str);
+}
+
+static gboolean
+_gst_structure_parse_field (gchar *str, gchar **after, GstStructureField *field)
+{
+  gchar *name;
+  gchar *name_end;
+  gchar *s;
+  gchar c;
+
+  s = str;
+
+  while(g_ascii_isspace (*s)) s++;
+  name = s;
+  if (!_gst_structure_parse_simple_string (s, &name_end)) return FALSE;
+
+  s = name_end;
+  while(g_ascii_isspace (*s)) s++;
+
+  if (*s != '=') return FALSE;
+  s++;
+
+  c = *name_end;
+  *name_end = 0;
+  field->name = g_quark_from_string (name);
+  *name_end = c;
+
+  if (!_gst_structure_parse_value (s, &s, &field->value, G_TYPE_INVALID))
+    return FALSE;
+
+  *after = s;
+  return TRUE;
+}
+
+static gboolean
+_gst_structure_parse_value (gchar *str, gchar **after, GValue *value,
+    GType default_type)
+{
+  gchar *type_name;
+  gchar *type_end;
+  gchar *value_s;
+  gchar *value_end;
+  gchar *s;
+  gchar c;
+  int ret = 0;
+  GType type = default_type;
+
+
+  s = str;
+  while(g_ascii_isspace (*s)) s++;
+
+  type_name = NULL;
+  if (*s == '(') {
+    type = G_TYPE_INVALID;
+
+    s++;
+    while(g_ascii_isspace (*s)) s++;
+    type_name = s;
+    if (!_gst_structure_parse_simple_string (s, &type_end)) return FALSE;
+    s = type_end;
+    while(g_ascii_isspace (*s)) s++;
+    if (*s != ')') return FALSE;
+    s++;
+    while(g_ascii_isspace (*s)) s++;
+
+    c = *type_end;
+    *type_end = 0;
+    type = _gst_structure_from_abbr(type_name);
+    *type_end = c;
+
+    if (type == G_TYPE_INVALID) return FALSE;
+  }
+
+  while(g_ascii_isspace (*s)) s++;
   if (*s == '[') {
-    ret = _gst_structure_parse_range (s, &s, type, &field->value);
-  } else if (*s == '(') {
-    ret = _gst_structure_parse_list (s, &s, type, &field->value);
+    ret = _gst_structure_parse_range (s, &s, value, type);
+  } else if (*s == '{') {
+    ret = _gst_structure_parse_list (s, &s, value, type);
   } else {
-    ret = _gst_structure_parse_value(s, &s, type, &field->value);
+    value_s = s;
+    if (!_gst_structure_parse_string (s, &value_end, &s)) return FALSE;
+
+    c = *value_end;
+    *value_end = 0;
+    if (type == G_TYPE_INVALID) {
+      GType try_types[] = { G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_STRING };
+      int i;
+
+      for(i=0;i<3;i++) {
+	g_value_init(value, try_types[i]);
+	ret = gst_value_deserialize (value, value_s);
+	if (ret) break;
+	g_value_unset(value);
+      }
+    } else {
+      g_value_init(value, type);
+
+      ret = gst_value_deserialize (value, value_s);
+    }
+    *value_end = c;
   }
   
   *after = s;
@@ -1320,7 +1322,8 @@ g_print("type n is \"%s\"\n",g_type_name(type));
 
 /**
  * gst_structure_from_string:
- * @structure: a #GstStructure
+ * @string: a string representation of a #GstStructure.
+ * @end: FIXME, deduce from code
  *
  * Creates a #GstStructure from a string representation.
  * 
@@ -1364,7 +1367,6 @@ gst_structure_from_string (const gchar *string, gchar **end)
 
     memset(&field,0,sizeof(field));
     res = _gst_structure_parse_field (r, &r, &field);
-g_print("returned %d \"%s\"\n", res, r);
     if (!res) {
       gst_structure_free (structure);
       return NULL;
