@@ -26,26 +26,19 @@
 G_BEGIN_DECLS
 
 typedef struct _GstStructure GstStructure;
-typedef struct _GstStructureField GstStructureField;
 
-typedef void (*GstStructureForeachFunc) (GstStructure *structure,
-    GQuark field_id, GValue *value, gpointer user_data);
+typedef gboolean (*GstStructureForeachFunc) (GQuark field_id, GValue *value,
+    gpointer user_data);
 
 struct _GstStructure {
-  int len;
+  GType type;
 
   GQuark name;
 
   GArray *fields;
 };
 
-struct _GstStructureField {
-  GQuark name;
-  GValue value;
-};
-
-#define GST_STRUCTURE_FIELD(structure, index) \
-    &g_array_index((structure)->fields, GstStructureField, (index))
+#define GST_TYPE_STRUCTURE (gst_structure_get_type())
 
 GType gst_structure_get_type(void);
 void _gst_structure_initialize(void);
@@ -59,50 +52,48 @@ GstStructure *gst_structure_new_valist(const gchar *name,
 GstStructure *gst_structure_copy(GstStructure *structure);
 void gst_structure_free(GstStructure *structure);
 
-G_CONST_RETURN gchar *gst_structure_get_name(GstStructure *structure);
+G_CONST_RETURN gchar *gst_structure_get_name(const GstStructure *structure);
 void gst_structure_set_name(GstStructure *structure, const gchar *name);
-void gst_structure_set_field_copy (GstStructure *structure,
-    const GstStructureField *field);
-void gst_structure_set_field (GstStructure *structure,
-    GstStructureField *field);
 
 void gst_structure_id_set_value(GstStructure *structure, GQuark field,
     const GValue *value);
-void gst_structure_set_value(GstStructure *structure, const gchar *field,
+void gst_structure_set_value(GstStructure *structure, const gchar *fieldname,
     const GValue *value);
-void gst_structure_set(GstStructure *structure, const gchar *field, ...);
-void gst_structure_set_valist(GstStructure *structure, const gchar *field,
+void gst_structure_set(GstStructure *structure, const gchar *fieldname, ...);
+void gst_structure_set_valist(GstStructure *structure, const gchar *fieldname,
     va_list varargs);
+G_CONST_RETURN GValue *gst_structure_id_get_value(const GstStructure *structure,
+    GQuark field);
 G_CONST_RETURN GValue *gst_structure_get_value(const GstStructure *structure,
-    const gchar *field);
-GstStructureField *gst_structure_get_field(const GstStructure *structure,
-        const gchar *fieldname);
-GstStructureField *gst_structure_id_get_field(const GstStructure *structure,
-        GQuark fieldname);
-void gst_structure_remove_field(GstStructure *structure, const gchar *field);
+    const gchar *fieldname);
+void gst_structure_remove_field(GstStructure *structure, const gchar *fieldname);
+void gst_structure_remove_fields(GstStructure *structure, 
+    const gchar *fieldname, ...);
+void gst_structure_remove_fields_valist(GstStructure *structure, 
+    const gchar *fieldname, va_list varargs);
 void gst_structure_remove_all_fields(GstStructure *structure);
 
 GType gst_structure_get_field_type(const GstStructure *structure,
-    const gchar *field);
-void gst_structure_field_foreach (GstStructure *structure,
+    const gchar *fieldname);
+gboolean gst_structure_foreach (GstStructure *structure,
     GstStructureForeachFunc func, gpointer user_data);
 gint gst_structure_n_fields(const GstStructure *structure);
-gboolean gst_structure_has_field(const GstStructure *structure, const gchar *field);
+gboolean gst_structure_has_field(const GstStructure *structure, const gchar *fieldname);
 gboolean gst_structure_has_field_typed(const GstStructure *structure,
-    const gchar *field, GType type);
+    const gchar *fieldname, GType type);
 
 /* utility functions */
 
-gboolean gst_structure_get_boolean(const GstStructure *structure, const gchar *field,
+gboolean gst_structure_get_boolean(const GstStructure *structure, const gchar *fieldname,
     gboolean *value);
-gboolean gst_structure_get_int(const GstStructure *structure, const gchar *field,
+gboolean gst_structure_get_int(const GstStructure *structure, const gchar *fieldname,
     gint *value);
-gboolean gst_structure_get_fourcc(const GstStructure *structure, const gchar *field,
+gboolean gst_structure_get_fourcc(const GstStructure *structure, const gchar *fieldname,
     guint32 *value);
-gboolean gst_structure_get_double(const GstStructure *structure, const gchar *field,
+gboolean gst_structure_get_double(const GstStructure *structure, const gchar *fieldname,
     gdouble *value);
 G_CONST_RETURN gchar *gst_structure_get_string(const GstStructure *structure,
-    const gchar *field);
+    const gchar *fieldname);
 
 gchar * gst_structure_to_string(const GstStructure *structure);
 GstStructure * gst_structure_from_string (const gchar *string, gchar **end);
