@@ -24,6 +24,7 @@
 #endif
 
 #include "gst_private.h"
+#include "gst-i18n-lib.h"
 #include "gsttag.h"
 #include "gstinfo.h"
 #include "gstvalue.h"
@@ -40,6 +41,7 @@ typedef struct {
   gchar *		blurb;		/* translated description of type */
 
   GstTagMergeFunc	merge_func;	/* functions to merge the values */
+  GstTagFlag            flag;           /* type of tag */
 } GstTagInfo;
 
 #define TAGLIST "taglist"
@@ -55,116 +57,156 @@ _gst_tag_initialize (void)
   gst_tag_list_quark = g_quark_from_static_string (TAGLIST);
   __tag_mutex = g_mutex_new ();
   __tags = g_hash_table_new (g_direct_hash, g_direct_equal);
-  gst_tag_register (GST_TAG_TITLE,	      
+  gst_tag_register (GST_TAG_TITLE, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("title"),
 		    _("commonly used title"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_ARTIST,
+  gst_tag_register (GST_TAG_ARTIST, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("artist"),
-		    _("person(s) resposible for the recording"),
+		    _("person(s) responsible for the recording"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_ALBUM,
+  gst_tag_register (GST_TAG_ALBUM, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("album"),
 		    _("album containing this data"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_DATE,
+  gst_tag_register (GST_TAG_DATE, GST_TAG_FLAG_META,
 		    G_TYPE_UINT, /* FIXME: own data type for dates? */
 		    _("date"),
-		    _("date the data was created in julien days"),
+		    _("date the data was created (in Julian calendar days)"),
 		    NULL);
-  gst_tag_register (GST_TAG_GENRE,
+  gst_tag_register (GST_TAG_GENRE, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("genre"), 
 		    _("genre this data belongs to"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_COMMENT,
+  gst_tag_register (GST_TAG_COMMENT, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("comment"),
 		    _("free text commenting the data"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_TRACK_NUMBER,
+  gst_tag_register (GST_TAG_TRACK_NUMBER, GST_TAG_FLAG_META,
 		    G_TYPE_UINT,
 		    _("track number"),
 		    _("track number inside a collection"),
 		    gst_tag_merge_use_first);
-  gst_tag_register (GST_TAG_TRACK_COUNT,
+  gst_tag_register (GST_TAG_TRACK_COUNT, GST_TAG_FLAG_META,
 		    G_TYPE_UINT,
 		    _("track count"),
 		    _("count of tracks inside collection this track belongs to"), 
 		    gst_tag_merge_use_first);
-  gst_tag_register (GST_TAG_LOCATION,
+  gst_tag_register (GST_TAG_LOCATION, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
-		    _("loccation"),
+		    _("location"),
 		    _("original location of file as a URI"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_DESCRIPTION,
+  gst_tag_register (GST_TAG_DESCRIPTION, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("description"),
 		    _("short text describing the content of the data"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_VERSION,
+  gst_tag_register (GST_TAG_VERSION, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("version"),
 		    _("version of this data"),
 		    NULL);
-  gst_tag_register (GST_TAG_ISRC,
+  gst_tag_register (GST_TAG_ISRC, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("ISRC"),
 		    _("International Standard Recording Code - see http://www.ifpi.org/isrc/"),
 		    NULL);
-  gst_tag_register (GST_TAG_ORGANIZATION,
+  gst_tag_register (GST_TAG_ORGANIZATION, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("organization"),
 		    _("organization"), /* FIXME */
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_COPYRIGHT,
+  gst_tag_register (GST_TAG_COPYRIGHT, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("copyright"),
 		    _("copyright notice of the data"),
 		    NULL);
-  gst_tag_register (GST_TAG_CONTACT,
+  gst_tag_register (GST_TAG_CONTACT, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("contact"),
 		    _("contact information"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_LICENSE,	
+  gst_tag_register (GST_TAG_LICENSE,	 GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("license"),
 		    _("license of data"),
 		    NULL);
-  gst_tag_register (GST_TAG_PERFORMER,
+  gst_tag_register (GST_TAG_PERFORMER, GST_TAG_FLAG_META,
 		    G_TYPE_STRING,
 		    _("performer"),
 		    _("person(s) performing"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_DURATION,
+  gst_tag_register (GST_TAG_DURATION, GST_TAG_FLAG_DECODED,
 		    G_TYPE_UINT64,
 		    _("duration"),
 		    _("length in GStreamer time units (nanoseconds)"),
 		    NULL);
-  gst_tag_register (GST_TAG_CODEC,
+  gst_tag_register (GST_TAG_CODEC, GST_TAG_FLAG_ENCODED,
 		    G_TYPE_STRING,
 		    _("codec"),
 		    _("codec the data is stored in"),
 		    gst_tag_merge_strings_with_comma);
-  gst_tag_register (GST_TAG_MINIMUM_BITRATE,
-		    G_TYPE_UINT,
-		    _("minimum bitrate"),
-		    _("minimum bitrate in bits/s"),
-		    NULL);
-  gst_tag_register (GST_TAG_BITRATE,
+  gst_tag_register (GST_TAG_BITRATE, GST_TAG_FLAG_ENCODED,
 		    G_TYPE_UINT,
 		    _("bitrate"),
 		    _("exact or average bitrate in bits/s"),
 		    NULL);
-  gst_tag_register (GST_TAG_MAXIMUM_BITRATE,
+  gst_tag_register (GST_TAG_NOMINAL_BITRATE, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_UINT,
+		    _("nominal bitrate"),
+		    _("nominal bitrate in bits/s"),
+		    NULL);
+  gst_tag_register (GST_TAG_MINIMUM_BITRATE, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_UINT,
+		    _("minimum bitrate"),
+		    _("minimum bitrate in bits/s"),
+		    NULL);
+  gst_tag_register (GST_TAG_MAXIMUM_BITRATE, GST_TAG_FLAG_ENCODED,
 		    G_TYPE_UINT,
 		    _("maximum bitrate"),
 		    _("maximum bitrate in bits/s"),
 		    NULL);
+  gst_tag_register (GST_TAG_ENCODER, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_STRING,
+		    _("encoder"),
+		    _("encoder used to encode this stream"),
+		    NULL);
+  gst_tag_register (GST_TAG_ENCODER_VERSION, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_UINT,
+		    _("encoder version"),
+		    _("version of the encoder used to encode this stream"),
+		    NULL);
+  gst_tag_register (GST_TAG_SERIAL, GST_TAG_FLAG_ENCODED,
+		    G_TYPE_UINT,
+		    _("serial"),
+		    _("serial number of track"),
+		    NULL);
+  gst_tag_register (GST_TAG_TRACK_GAIN, GST_TAG_FLAG_META,
+                    G_TYPE_DOUBLE,
+		    _("replaygain track gain"),
+		    _("track gain in db"),
+		    NULL);
+  gst_tag_register (GST_TAG_TRACK_PEAK, GST_TAG_FLAG_META,
+                    G_TYPE_DOUBLE,
+		    _("replaygain track peak"),
+		    _("peak of the track"),
+		    NULL);
+  gst_tag_register (GST_TAG_ALBUM_GAIN, GST_TAG_FLAG_META,
+                    G_TYPE_DOUBLE,
+		    _("replaygain album gain"),
+		    _("album gain in db"),
+		    NULL);
+  gst_tag_register (GST_TAG_ALBUM_PEAK, GST_TAG_FLAG_META,
+                    G_TYPE_DOUBLE,
+		    _("replaygain album peak"),
+		    _("peak of the album"),
+		    NULL);
 }
 /**
  * gst_tag_merge_use_first:
@@ -223,6 +265,7 @@ gst_tag_lookup (GQuark entry)
 /**
  * gst_tag_register:
  * @name: the name or identifier string
+ * @flag: a flag describing the type of tag info
  * @type: the type this data is in
  * @nick: human-readable name
  * @blurb: a human-readable description about this tag
@@ -231,13 +274,12 @@ gst_tag_lookup (GQuark entry)
  * Registers a new tag type for the use with GStreamer's type system. If a type
  * with that name is already registered, that one is used.
  * The old registration may have used a different type however. So don't rely
- * on youre supplied values.
- * If you know the type is already registered, use gst_tag_lookup instead.
+ * on your supplied values.
  * This function takes ownership of all supplied variables.
  */
 void
-gst_tag_register (gchar *name, GType type, gchar *nick, gchar *blurb,
-		  GstTagMergeFunc func)
+gst_tag_register (gchar *name, GstTagFlag flag, GType type,
+                  gchar *nick, gchar *blurb, GstTagMergeFunc func)
 {
   GQuark key;
   GstTagInfo *info;
@@ -252,6 +294,7 @@ gst_tag_register (gchar *name, GType type, gchar *nick, gchar *blurb,
   g_return_if_fail (info == NULL);
   
   info = g_new (GstTagInfo, 1);
+  info->flag = flag;
   info->type = type;
   info->nick = nick;
   info->blurb = blurb;
@@ -390,26 +433,26 @@ static void
 gst_tag_list_add_value_internal (GstStructure *list, GstTagMergeMode mode, GQuark tag, GValue *value)
 {
   GstTagInfo *info = gst_tag_lookup (tag);
-  GstStructureField *field;
+  const GValue *value2;
   
   g_assert (info != NULL);
 
-  if (info->merge_func && (field = gst_structure_id_get_field (list, tag)) != NULL) {
-    GValue value2 = { 0, };
+  if (info->merge_func && (value2 = gst_structure_id_get_value (list, tag)) != NULL) {
+    GValue dest = { 0, };
     switch (mode) {
       case GST_TAG_MERGE_REPLACE_ALL:
       case GST_TAG_MERGE_REPLACE:
 	gst_structure_id_set_value (list, tag, value);
 	break;
       case GST_TAG_MERGE_PREPEND:
-	gst_value_list_concat (&value2, value, &field->value);
-	gst_structure_id_set_value (list, tag, &value2);
-	g_value_unset (&value2);
+	gst_value_list_concat (&dest, value, value2);
+	gst_structure_id_set_value (list, tag, &dest);
+	g_value_unset (&dest);
 	break;
       case GST_TAG_MERGE_APPEND:
-	gst_value_list_concat (&value2, &field->value, value);
-	gst_structure_id_set_value (list, tag, &value2);
-	g_value_unset (&value2);
+	gst_value_list_concat (&dest, value2, value);
+	gst_structure_id_set_value (list, tag, &dest);
+	g_value_unset (&dest);
 	break;
       case GST_TAG_MERGE_KEEP:
       case GST_TAG_MERGE_KEEP_ALL:
@@ -422,7 +465,7 @@ gst_tag_list_add_value_internal (GstStructure *list, GstTagMergeMode mode, GQuar
     switch (mode) {
       case GST_TAG_MERGE_APPEND:
       case GST_TAG_MERGE_KEEP:
-	if (gst_structure_id_get_field (list, tag) != NULL)
+	if (gst_structure_id_get_value (list, tag) != NULL)
 	  break;
 	/* fall through */
       case GST_TAG_MERGE_REPLACE_ALL:
@@ -438,12 +481,14 @@ gst_tag_list_add_value_internal (GstStructure *list, GstTagMergeMode mode, GQuar
     }
   }
 }
-static void
-gst_tag_list_copy_foreach (GstStructure *structure, GQuark tag, GValue *value, gpointer user_data)
+static gboolean
+gst_tag_list_copy_foreach (GQuark tag, GValue *value, gpointer user_data)
 {
   GstTagCopyData *copy = (GstTagCopyData *) user_data;
 
   gst_tag_list_add_value_internal (copy->list, copy->mode, tag, value);
+
+  return TRUE;
 }
 /**
  * gst_tag_list_insert:
@@ -467,7 +512,7 @@ gst_tag_list_insert (GstTagList *into, const GstTagList *from, GstTagMergeMode m
   if (mode == GST_TAG_MERGE_REPLACE_ALL) {
     gst_structure_remove_all_fields (data.list);
   }
-  gst_structure_field_foreach ((GstStructure *) from, gst_tag_list_copy_foreach, &data);
+  gst_structure_foreach ((GstStructure *) from, gst_tag_list_copy_foreach, &data);
 }
 /**
  * gst_tag_list_copy:
@@ -557,7 +602,7 @@ gst_tag_list_get_tag_size (const GstTagList *list, const gchar *tag)
  * @list: list to set tags in
  * @mode: the mode to use
  * @tag: tag
- * @...: values to set
+ * @...: NULL-terminated list of values to set
  *
  * Sets the values for the given tags using the specified mode.
  */
@@ -574,6 +619,28 @@ gst_tag_list_add (GstTagList *list, GstTagMergeMode mode, const gchar *tag, ...)
   gst_tag_list_add_valist (list, mode, tag, args);
   va_end (args);
 }
+/**
+ * gst_tag_list_add_values:
+ * @list: list to set tags in
+ * @mode: the mode to use
+ * @tag: tag
+ * @...: GValues to set
+ *
+ * Sets the GValues for the given tags using the specified mode.
+ */
+void
+gst_tag_list_add_values (GstTagList *list, GstTagMergeMode mode, const gchar *tag, ...)
+{
+  va_list args;
+
+  g_return_if_fail (GST_IS_TAG_LIST (list));
+  g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
+  g_return_if_fail (tag != NULL);
+  
+  va_start (args, tag);
+  gst_tag_list_add_valist_values (list, mode, tag, args);
+  va_end (args);
+}
 /**
  * gst_tag_list_add_valist:
  * @list: list to set tags in
@@ -598,6 +665,8 @@ gst_tag_list_add_valist (GstTagList *list, GstTagMergeMode mode, const gchar *ta
     GValue value = { 0, };
     quark = g_quark_from_string (tag);
     info = gst_tag_lookup (quark);
+    if (info == NULL)
+      g_warning ("no GstTag for %s", tag);
     g_return_if_fail (info != NULL);
     g_value_init (&value, info->type);
     G_VALUE_COLLECT (&value, var_args, 0, &error);
@@ -614,6 +683,33 @@ gst_tag_list_add_valist (GstTagList *list, GstTagMergeMode mode, const gchar *ta
     tag = va_arg (var_args, gchar *);
   }
 }
+/**
+ * gst_tag_list_add_valist_values:
+ * @list: list to set tags in
+ * @mode: the mode to use
+ * @tag: tag
+ * @var_args: tag / GValue pairs to set
+ *
+ * Sets the GValues for the given tags using the specified mode.
+ */
+void
+gst_tag_list_add_valist_values (GstTagList *list, GstTagMergeMode mode, const gchar *tag, va_list var_args)
+{
+  GstTagInfo *info;
+  GQuark quark;
+  
+  g_return_if_fail (GST_IS_TAG_LIST (list));
+  g_return_if_fail (GST_TAG_MODE_IS_VALID (mode));
+  g_return_if_fail (tag != NULL);
+  
+  while (tag != NULL) {
+    quark = g_quark_from_string (tag);
+    info = gst_tag_lookup (quark);
+    g_return_if_fail (info != NULL);
+    gst_tag_list_add_value_internal (list, mode, quark, va_arg (var_args, GValue *));
+    tag = va_arg (var_args, gchar *);
+  }
+}
 /**
  * gst_tag_list_remove_tag:
  * @list: list to remove tag from
@@ -631,14 +727,16 @@ gst_tag_list_remove_tag (GstTagList *list, const gchar *tag)
 }
 typedef struct {
   GstTagForeachFunc	func;
+  GstTagList *          tag_list;
   gpointer		data;
 } TagForeachData;
-static void 
-structure_foreach_wrapper (GstStructure *structure, GQuark field_id, 
+static int
+structure_foreach_wrapper (GQuark field_id, 
 	GValue *value, gpointer user_data)
 {
   TagForeachData *data = (TagForeachData *) user_data;
-  data->func (GST_TAG_LIST (structure), g_quark_to_string (field_id), data->data);
+  data->func (data->tag_list, g_quark_to_string (field_id), data->data);
+  return TRUE;
 }
 /**
  * gst_tag_list_foreach:
@@ -658,8 +756,9 @@ gst_tag_list_foreach (GstTagList *list, GstTagForeachFunc func, gpointer user_da
   g_return_if_fail (func != NULL);
   
   data.func = func;
+  data.tag_list = list;
   data.data = user_data;
-  gst_structure_field_foreach ((GstStructure *) list, structure_foreach_wrapper, &data);
+  gst_structure_foreach ((GstStructure *) list, structure_foreach_wrapper, &data);
 }
 
 /***** tag events *****/
